/* arptool
 *
 * by scut of teso,
 * based on libnet samples by route
 *
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <libnet.h>


#define	AUTHOR	"scut of teso"
#define	VERSION	"0.0.1"


u_char	enet_src[6] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
u_char	enet_dst[6] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
u_char	ip_src[4] = { 0x01, 0x01, 0x01, 0x01 };
u_char	ip_dst[4] = { 0x00, 0x00, 0x00, 0x00 };

char *		device = NULL;
 
void	usage (void);
int	arp_tell (struct libnet_link_int *l, u_char *device);
int	arp_bcast (struct libnet_link_int *l, u_char *device);
int	ip_transform (char *ip, u_char *ip_ar);
int	mac_transform (char *mac, u_char *mac_ar);


int
mac_transform (char *mac, u_char *mac_ar)
{
	unsigned int	foo[6];

	if (sscanf (mac, "%2x:%2x:%2x:%2x:%2x:%2x",
		&foo[0], &foo[1], &foo[2], &foo[3], &foo[4], &foo[5]) != 6)
	{
		return (1);
	}

	mac_ar[0] = foo[0];
	mac_ar[1] = foo[1];
	mac_ar[2] = foo[2];
	mac_ar[3] = foo[3];
	mac_ar[4] = foo[4];
	mac_ar[5] = foo[5];

	return (0);
}


int
ip_transform (char *ip, u_char *ip_ar)
{
	unsigned short int	foo[4];
	if (sscanf (ip, "%hu.%hu.%hu.%hu", &foo[0], &foo[1], &foo[2], &foo[3]) != 4)
		return (1);
	ip_ar[0] = foo[0];
	ip_ar[1] = foo[1];
	ip_ar[2] = foo[2];
	ip_ar[3] = foo[3];

	return (0);
}


void
usage (void)
{
	printf ("usage: arptool <device> <type> <dstip> <dstmac> <srcip> <srcmac>\n\n"
		"<device>\t\ttarget device to send arp packet on\n"
		"<type>\t\t\ttype of arp packet to send\n"
		"\ttell\tarp reply\n"
		"\tbcast\tarp who-has ask\n"
		"<dstip>/<dstmac>\tthe destination pair\n"
		"<srcip>/<srcmac>\tthe source pair\n\n");

	exit (EXIT_SUCCESS);
}


int
main (int argc, char **argv)
{
	char				c;
	char				errbuf[256];
	struct libnet_link_int *	l;
	char *				type;


	device = argv[1];

	if (argc != 7 || device == NULL ||
		(strcasecmp (argv[2], "tell") != 0 && strcasecmp (argv[2], "bcast") != 0))
	{
		usage ();
	}

	type = argv[2];
	if (ip_transform (argv[3], ip_dst) ||
		ip_transform (argv[5], ip_src) ||
		mac_transform (argv[4], enet_dst) ||
		mac_transform (argv[6], enet_src))
	{
		fprintf (stderr, "translation failed.\n");
		exit (EXIT_FAILURE);
	}


	l = libnet_open_link_interface (device, errbuf);
	if (l == NULL) {
		fprintf (stderr, "libnet_open_link_interface: %s\n", errbuf);

		exit (EXIT_FAILURE);
	}

	if (strcasecmp (type, "bcast") == 0) {
		c = arp_bcast (l, device);
	} else {
		c = arp_tell (l, device);
	}

	if (c == -1)
		exit (EXIT_FAILURE);

	exit (EXIT_SUCCESS);
}


int
arp_tell (struct libnet_link_int *l, u_char *device)
{
	int		n;
	u_char *	buf;

	if (libnet_init_packet (LIBNET_ARP_H + LIBNET_ETH_H, &buf) == -1) {
		perror ("libnet_init_packet memory:");
		exit (EXIT_FAILURE);
	}

	libnet_build_ethernet (enet_dst, enet_src, ETHERTYPE_ARP, NULL, 0, buf);

	libnet_build_arp (ARPHRD_ETHER,
		ETHERTYPE_IP,
		6,
		4,
		ARPOP_REPLY,
		enet_src,
		ip_src,
		enet_dst,
		ip_dst,
		NULL,
		0,
		buf + LIBNET_ETH_H);

	n = libnet_write_link_layer (l, device, buf, LIBNET_ARP_H + LIBNET_ETH_H);

	printf ("%3d ARP tell on linktype %d\n", n, l->linktype);

	libnet_destroy_packet (&buf);

	return (n);
}


int
arp_bcast (struct libnet_link_int *l, u_char *device)
{
	int		n;
	u_char *	buf;

	if (libnet_init_packet (LIBNET_ARP_H + LIBNET_ETH_H, &buf) == -1) {
		perror ("libnet_init_packet memory:");
		exit (EXIT_FAILURE);
	}

	libnet_build_ethernet (enet_dst, enet_src, ETHERTYPE_ARP, NULL, 0, buf);

	libnet_build_arp (ARPHRD_ETHER,
		ETHERTYPE_IP,
		6,
		4,
		ARPOP_REQUEST,
		enet_src,
		ip_src,
		enet_dst,
		ip_dst,
		NULL,
		0,
		buf + LIBNET_ETH_H);

	n = libnet_write_link_layer (l, device, buf, LIBNET_ARP_H + LIBNET_ETH_H);

	printf ("%3d ARP bcast on linktype %d\n", n, l->linktype);

	libnet_destroy_packet (&buf);

	return (n);
}

